import { NestFactory } from '@nestjs/core';
import { SampleAppModule } from './sample-app.module';
import { MicroserviceOptions, Transport } from '@nestjs/microservices';
import { ConfigService } from '@nestjs/config';
import 'dotenv/config'
import { Logger } from '@nestjs/common';

const mainLogger = new Logger(`Sample-App`)
async function bootstrap() {
  // This example contains a hybrid application (HTTP + TCP + Socket)
  const app = await NestFactory.create(SampleAppModule);
  const config: ConfigService = app.get(ConfigService)

  const host: string = config.get<string>(`sampleApp.host`) as string
  const restPort: number = config.get<number>(`sampleApp.restPort`) as number
  const tcpPort: number = config.get<number>(`sampleApp.tcpPort`) as number

  app.enableCors({
    origin: `*`, // specify the allowed origin(s)
    methods: 'GET,POST,PUT,DELETE,OPTIONS',
    allowedHeaders: 'Content-Type, Authorization',
    credentials: true, // if you need to support cookies or other credentials
  });

  app.connectMicroservice<MicroserviceOptions>({
    transport: Transport.TCP,
    options: {
      host: host,
      port: tcpPort,
      retryAttempts: 5,
      retryDelay: 3000
    },
  });

  await app.startAllMicroservices();
  await app.listen(restPort, host);
  mainLogger.log(`REST API is running on: ${await app.getUrl()} and TCP is running on ${host}:${tcpPort} and Socket is running on ${host}:${restPort}`);
}
bootstrap();
