import { Inject, Injectable, Logger } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';
import { ConnectedSocket, MessageBody, OnGatewayConnection, OnGatewayDisconnect, SubscribeMessage, WebSocketGateway, WebSocketServer, WsResponse, } from '@nestjs/websockets';
import { from, Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Server, Socket } from 'socket.io';
import { SampleAppService } from './sample-app.service';

@WebSocketGateway({
    cors: {
        origin: '*',
    },
    namespace: `Sample-App`
})
export class SampleEventsGateway implements OnGatewayConnection, OnGatewayDisconnect {
    @WebSocketServer()
    server!: Server;
    logger: Logger = new Logger(`SampleEventGateway`)

    constructor(private readonly service: SampleAppService) {
        // logic here
    }

    @SubscribeMessage('test')
    test(@MessageBody() data: any, @ConnectedSocket() client: Socket): Observable<WsResponse<any>> {
        throw new Error(`Method not implemented`)
    }

    @SubscribeMessage('message')
    handleMesage(@MessageBody() data: string, @ConnectedSocket() client: Socket): void {
        // console.log(data)
        this.logger.log(`Received data from ${client.id}`)
        this.service.emit(data)
    }

    // this two methods are non essential. as they are used to keep track of the local ui port 3000, not keeping track of the actual clients 
    handleConnection(client: any) {
        this.logger.log(`Socket Client ${client.id} connected`);
    }

    handleDisconnect(client: any) {
        this.logger.log(`Socket Client ${client.id} disconnected`);
    }
}